/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Preferences Toolbar 3.
 *
 * The Initial Developer of the Original Code is
 * Manuel Reimer.
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Manuel Reimer <Manuel.Reimer@gmx.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


var goPrefBar = null;

const ImportType_Import = 1
const ImportType_Update = 2
const ImportType_Reset = 3

function Init(go) {
    goPrefBar = go;

    if (goPrefBar.RDF.isEmpty) {
        doUpdate();
        navigator.preference("extensions.prefbar.version", goPrefBar.prefbarVersion);
    }
    else
        prefbarCheckForUpdate();
}

function Export(win, toexport, filename) {
    var RDF = goPrefBar.RDF.RDFService;
    var mDatasource = goPrefBar.RDF.mDatasource;

    if (toexport.length == 0) return;

    var fphandler = Components.classes["@mozilla.org/network/protocol;1?name=file"].getService(Components.interfaces.nsIFileProtocolHandler);
    var exportfileobj = fphandler.getFileFromURLSpec(filename);

    // Write Template first. If this fails, then throw error (readonly?)
    try {
        goPrefBar.RDF.WriteBTNTemplate(exportfileobj);
    }
    catch(e) {
	win.alert(goPrefBar.GetString("importexport.properties", "exporterrreadonly"));
	return;
    }

    var expDatasource = RDF.GetDataSourceBlocking(filename);

    var RDFCU = Components.classes['@mozilla.org/rdf/container-utils;1'].getService();
    RDFCU = RDFCU.QueryInterface(Components.interfaces.nsIRDFContainerUtils);

    var CCache = new Array();

    for (var i = 0; i < toexport.length; i++) {
        var RitemId = RDF.GetResource(toexport[i]);

        var parents = getParents(mDatasource, RitemId);
        for (var group in parents) {
            var menuid = parents[group];
            if (menuid != "disabled") menuid = "enabled";

            var CName = "urn:prefbar:" + group + ":" + menuid;
            var RCName = RDF.GetResource(CName);

            var Container;
            if (!(CName in CCache)) {
                CCache[CName] = RDFCU.MakeSeq(expDatasource, RCName);
            }
            Container = CCache[CName];

            Container.AppendElement(RitemId);
        }

        var properties = mDatasource.ArcLabelsOut(RitemId);
	while(properties.hasMoreElements()) {
	    var Rproperty = properties.getNext();

	    var Nvalue = mDatasource.GetTarget(RitemId, Rproperty, true);

	    expDatasource.Assert(RitemId, Rproperty, Nvalue, true);
	}

    }

    var remoteControll = expDatasource.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource);
    remoteControll.Flush();

    RDF.UnregisterDataSource(expDatasource);

    /*
    //****HACK**** (Mozilla Bug 236920) Check if the file was saved on a disc which didn't have enough space free.
    //             Sometimes flush will fill the whole disc up to the last byte and sometimes it simply
    //             doesn't save anything. We check for both.
    if (exportfileobj.fileSize == 0) {
	win.alert(goPrefBar.GetString("importexport.properties", "exporterrfailedwrite"));
	return;
    }

    var fileurlpath = filename.substr(0, filename.lastIndexOf("/") + 1);
    var temppathobj = fphandler.getFileFromURLSpec(fileurlpath);
    temppathobj.append(prefbarGetTemporaryFile(temppathobj, "prefbar", ".tmp"));
    var abyte = "x";
    try	{
	try {
	    temppathobj.create(0x00,0644);
	} catch (e) { }
	var stream = Components.classes['@mozilla.org/network/file-output-stream;1'].createInstance(Components.interfaces.nsIFileOutputStream);
	stream.init(temppathobj, 2, 0x200, false); // open as "write only"

	stream.write(abyte, abyte.length);
	stream.close();
    } catch (e) {
	win.alert(prefbarGetString("importexport.properties", "exporterrfailedwrite"));
	temppathobj.remove(false);
	return;
    }
    temppathobj.remove(false);
    //****END HACK****

    */
}

function Import(win, origfilename, ImportType, forceoverwrite) {
    var RDF = goPrefBar.RDF.RDFService;
    var mDatasource = goPrefBar.RDF.mDatasource;

    goPrefBar.dump("IMPORTER: Import started for " + origfilename);

    //****HACK**** (Mozilla Bug 237784) Copy to temporary file first
    // This hack only works if we have to handle the "file:"-protocol
    var filename = "";
    if(origfilename.toUpperCase().substr(0,5) == "FILE:") {
	var dirService = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties);
	var fileprotocolService = Components.classes["@mozilla.org/network/protocol;1?name=file"].getService(Components.interfaces.nsIFileProtocolHandler);

	var tempDir = dirService.get("TmpD", Components.interfaces.nsIFile);
	var strtmpfilename = prefbarGetTemporaryFile(tempDir, "prefbar", ".rdf");

	var origfileobj = fileprotocolService.getFileFromURLSpec(origfilename);
	origfileobj.copyTo(tempDir, strtmpfilename);

	var tempfileobj = tempDir.clone();
	tempfileobj.append(strtmpfilename);

	filename = fileprotocolService.getURLSpecFromFile(tempfileobj);
    }
    else
        filename = origfilename;
    //****END HACK****

    var askedUser = false;
    var Overwrite = false;
	
    if (forceoverwrite != undefined)
    {
	Overwrite = forceoverwrite;
	askedUser = true;
    }
    if (ImportType == undefined) ImportType = ImportType_Import;

    var impDatasource = RDF.GetDataSourceBlocking(filename);

    if (!goPrefBar.RDF.CanReadFormat(impDatasource)) {
        win.alert(goPrefBar.GetString("importexport.properties", "importerrversion").replace("$filename", origfilename));
        if(origfilename.toUpperCase().substr(0,5) == "FILE:")
            tempfileobj.remove(false);
        return false;
    }

    if (goPrefBar.RDF.FormatUpdateNeeded(impDatasource)) {
        /*
          We don't overwrite anything as long as we are forced to use a
          temporary file. Maybe this will change in future

        if (!win.confirm("The file \"" + origfilename + "\" had been created with an older PrefBar version. We have to update the format of that file before we are able to import. This will modify the file you want to import! Do you want to continue? (Fell free to backup the file before hitting \"Yes\").")) {
            if(origfilename.toUpperCase().substr(0,5) == "FILE:")
                tempfileobj.remove(false);
            return false;
        }
        */
        win.alert(goPrefBar.GetString("importexport.properties", "importinfooldformat").replace("$filename", origfilename));
        goPrefBar.RDF.PerformFormatUpdates(impDatasource);
    }

    var RDFCU = Components.classes['@mozilla.org/rdf/container-utils;1'].getService();
    RDFCU = RDFCU.QueryInterface(Components.interfaces.nsIRDFContainerUtils);

    var Classes = new Array("browserbuttons");
    var Menus = new Array("enabled", "disabled");

    // Read all possible Containers from btn file
    var ImpCCache = new Array();
    for (var clsid in Classes) {
        var cls = Classes[clsid];
        for (var mnuid in Menus) {
            var mnu = Menus[mnuid];
            var CurMnu = "urn:prefbar:" + cls + ":" + mnu;
            var RCurMnu = RDF.GetResource(CurMnu);
            if (impDatasource.ArcLabelsOut(RCurMnu).hasMoreElements())
                ImpCCache[CurMnu] = RDFCU.MakeSeq(impDatasource, RCurMnu);
        }
    }

    // Read all possible item ids from btn file
    var ImpItems = new Array();
    for (var CName in ImpCCache) {
        var Container = ImpCCache[CName];
        var impelements = Container.GetElements();
        while(impelements.hasMoreElements()) {
            var impelement = impelements.getNext();
            ImpItems[impelement.QueryInterface(Components.interfaces.nsIRDFResource).Value] = true;
        }
    }

    var MCCache = new Array();

    for (var ImpItem in ImpItems) {
        var RImpItem = RDF.GetResource(ImpItem);

        var IdFree = !mDatasource.ArcLabelsOut(RImpItem).hasMoreElements();

	// Read the "dontupdatelistitems"-tag for the current item.
	var DontUpdateListItems = false;
	var DULattribute = RDF.GetResource("http://www.xulplanet.com/rdf/prefbar#dontupdatelistitems");
	var DULvalue = impDatasource.GetTarget(RImpItem, DULattribute, true);
	if (DULvalue)
	    DontUpdateListItems = DULvalue.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
	//alert(DontUpdateListItems);

	goPrefBar.dump("IMPORTER: -> Current Item is: " + ImpItem);
	goPrefBar.dump("IMPORTER: IdFree: " + IdFree);

	// If this item would overwrite existing Items, then
	if (!IdFree) {
	    // If the User wasn't asked before, then ask if he wants to
	    // overwrite this items
	    if(!askedUser) {
		Overwrite = win.confirm(goPrefBar.GetString("importexport.properties", "importquestionoverwrite"));
		askedUser = true;
	    }
	    // If the user has chosen to not overwrite, then continue with the
            // next Button
	    if(!Overwrite) continue;
	}


	// If the Id is not free
	if (!IdFree) {

            //remove all old properties
	    var oldproperties = mDatasource.ArcLabelsOut(impelement);
	    while(oldproperties.hasMoreElements()) {
		var Roldproperty = oldproperties.getNext();

		var OldPropertyString = Roldproperty.QueryInterface(Components.interfaces.nsIRDFResource).Value.toUpperCase();

		// Check if we are allowed to overwrite this property. If not, then continue
		if (!Import_AllowOverwrite(ImportType, OldPropertyString, DontUpdateListItems))
		    continue;

		var Loldvalue = mDatasource.GetTarget(impelement, Roldproperty, true);
		mDatasource.Unassert(impelement, Roldproperty, Loldvalue);
	    }

            var impparents = getParents(impDatasource, RImpItem);
            var mparents = getParents(mDatasource, RImpItem);

            // Add item to groups defined in input file
            for (var impgroup in impparents) {
                var impmenu = impparents[impgroup];
                var CName = "urn:prefbar:" + impgroup + ":" + impmenu;
                var RCName = RDF.GetResource(CName);
                // Only valid container names!
                if (!(CName in ImpCCache)) continue;
                if (!(impgroup in mparents)) {
                    if (!(CName in MCCache)) {
                        MCCache[CName] = RDFCU.MakeSeq(mDatasource, RCName);
                    }
                    MCCache[CName].AppendElement(RImpItem);
                }
            }

            // Remove item from groups not defined in input file
            for (var mgroup in mparents) {
                var mmenu = mparents[mgroup];
                var CName = "urn:prefbar:" + mgroup + ":" + mmenu;
                var RCName = RDF.GetResource(CName);
                if (!(mgroup in impparents)) {
                    if (!(CName in MCCache)) {
                        MCCache[CName] = RDFCU.MakeSeq(mDatasource, RCName);
                    }
                    MCCache[CName].RemoveElement(RImpItem, true);
                }
            }
        }
	// If the Id doesn't exist, then create it
	else {
            var impparents = getParents(impDatasource, RImpItem);

            for (var impgroup in impparents) {
                var impmenu = impparents[impgroup];
                var CName = "urn:prefbar:" + impgroup + ":" + impmenu;
                var RCName = RDF.GetResource(CName);
                // Only valid container names!
                if (!(CName in ImpCCache)) continue;
                if (!(CName in MCCache)) {
                    MCCache[CName] = RDFCU.MakeSeq(mDatasource, RCName);
                }
                MCCache[CName].AppendElement(RImpItem);
            }
	}


	//Add all new properties
	var impproperties = impDatasource.ArcLabelsOut(RImpItem);
	while(impproperties.hasMoreElements()) {
	    var Rimpproperty = impproperties.getNext();
			
	    var ImpPropertyString = Rimpproperty.QueryInterface(Components.interfaces.nsIRDFResource).Value.toUpperCase();
	    // If the current Item is not a new item (overwrites/updates existing one)
	    if (!IdFree) {
				
		// Check if we are allowed to overwrite this property (if not
                // then it hasn't been deleted before and still has the old
                // value). If not, then continue
		if (!Import_AllowOverwrite(ImportType, ImpPropertyString, DontUpdateListItems))
		    continue;
	    }

	    var Nvalue = impDatasource.GetTarget(RImpItem, Rimpproperty, true);

            // If we have to write the label or optionlabel, then try to get a
            // localized version.
            if (ImpPropertyString.match(/PREFBAR#(LABEL)$/) ||
                ImpPropertyString.match(/PREFBAR#(OPTIONLABEL\d+)$/)) {
                try {
                    var pref=RegExp.$1.toLowerCase();
                    var id = ImpItem.replace(/urn:prefbar:buttons:/, "");
                    var localelabel = goPrefBar.GetString("buttons.properties", id + "." + pref);
                    Nvalue = RDF.GetLiteral(localelabel);
                } catch(e) {}
            }

	    mDatasource.Assert(RImpItem, Rimpproperty, Nvalue, true);
	}
    }

    goPrefBar.RDF.mDatasource.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();

    //****HACK**** (Mozilla Bug 237784) Delete Temporary file
    // This hack only works if we have to handle the "file:"-protocol
    if(origfilename.toUpperCase().substr(0,5) == "FILE:")
        tempfileobj.remove(false);
    //****END HACK****
}

function Import_AllowOverwrite(ImportType, Property, option_DontUpdateListItems) {
    // No need for additional checks if we reset. Everything has to be overwritten.
    if (ImportType == ImportType_Reset) return true;

    // **** LABEL PROPERTY ****
    if (Property == "HTTP://WWW.XULPLANET.COM/RDF/PREFBAR#LABEL") {
	// If we update, we don't touch the users labels
	if (ImportType == ImportType_Update)
	    return false;
    }

    // **** OPTIONLABEL/OPTIONVALUE PROPERTY ****
    if (Property.match(/^HTTP:\/\/WWW.XULPLANET.COM\/RDF\/PREFBAR#OPTION(LABEL|VALUE)\d+/)) {
	// We don't edit listitems, if the "DontUpdateListItems"-flag is "true"
	if (option_DontUpdateListItems)
	    return false;
    }

    return true;
}

function prefbarGetTemporaryFile(Folder, Prefix, Suffix) {
    var tempfile = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
    var counter = 1;
    var name = "";
    do {
	tempfile.initWithFile(Folder);
	name = Prefix + counter + Suffix;
	tempfile.append(name);
	counter++;
    } while(tempfile.exists())
	return name;
}

function prefbarCheckForUpdate() {
    var prefBranch = goPrefBar.PrefBranch;
    // Kill old Version property, if it exists
    if (prefBranch.prefHasUserValue("prefbar.version")) prefBranch.clearUserPref("prefbar.version");

    var savedVersion = navigator.preference("extensions.prefbar.version");
    if (savedVersion < goPrefBar.prefbarVersion || savedVersion == undefined) {
	var updatewanted = confirm(goPrefBar.GetString("importexport.properties", "updatewanted"));
	if (updatewanted) {
            doUpdate();
	    alert(goPrefBar.GetString("importexport.properties", "updatefinished"));
	}
	
	navigator.preference("extensions.prefbar.version", goPrefBar.prefbarVersion);
    }
}
function doUpdate() {
    Import(window, "chrome://prefbar/content/prefbar.rdf", ImportType_Update, true);
}

function getParents(datasource, RNode) {
    var RDF = goPrefBar.RDF.RDFService;
    var RDFCU = Components.classes['@mozilla.org/rdf/container-utils;1'].getService();
    RDFCU = RDFCU.QueryInterface(Components.interfaces.nsIRDFContainerUtils);

    var result = new Array();

    var arcsIn = datasource.ArcLabelsIn(RNode);
    while(arcsIn.hasMoreElements()) {
        var arc = arcsIn.getNext();
        if (arc instanceof Components.interfaces.nsIRDFResource) {
            if (RDFCU.IsOrdinalProperty(arc)) {
                var parent = datasource.GetSource(arc, RNode, true);
                var strparent = parent.QueryInterface(Components.interfaces.nsIRDFResource).Value;
                strparent.match(/^\w*\:\w*\:(\w*)\:(.*)/);
                result[RegExp.$1] = RegExp.$2;
            }
        }
    }
    return result;
}
