/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Preferences Toolbar 3.
 *
 * The Initial Developer of the Original Code is
 * Manuel Reimer.
 * Portions created by the Initial Developer are Copyright (C) 2___
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Aaron Andersen <aaron@xulplanet.com>
 *                 Kevin Teuscher
 *                 Manuel Reimer <Manuel.Reimer@gmx.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


// note: global vars in this file don't need "prefbar" prefix in their names

var prefbarActiveTree = null;
var allTree = null;
var enabledTree = null;

// some of the buttons in the middle of the panel
var enableButton = null;
var disableButton = null;
var moveUpButton = null
var moveDownButton = null;
var editButton = null;
var deleteButton = null;
var exportButton = null;

var RDF = null;
var mDatasource = null;

function prefbarSetupEverything() {
    parent.initPanel('chrome://prefbar/content/preferencesPanel.xul');
    // ensure the Advanced pref tree is open
    var advitem = parent.document.getElementById("advancedItem");
    if (advitem) advitem.setAttribute("open", "true");
    
    allTree = document.getElementById("allTree");
    enabledTree = document.getElementById("enabledTree");

    goPrefBar.RDF.AddmDatasource(allTree);
    goPrefBar.RDF.AddmDatasource(enabledTree);

    RDF = goPrefBar.RDF.RDFService;
    mDatasource = goPrefBar.RDF.mDatasource;

    enableButton = document.getElementById("itemEnable");
    disableButton = document.getElementById("itemDisable");
    moveUpButton = document.getElementById("itemMoveUp");
    moveDownButton = document.getElementById("itemMoveDown");
    editButton = document.getElementById("itemEdit");
    deleteButton = document.getElementById("itemDelete");
    exportButton = document.getElementById("itemExport");

    // Set the preset for the selections.
    enabledTree.focus();
    if (allTree.view) allTree.view.selection.select(-1);
    if (enabledTree.view) enabledTree.view.selection.select(0);
}

function prefbarInitEnabledTree() {
    //goPrefBar.dump("initenabled");
    // Remove selection from other tree
    // allTree.currentIndex = -1;       // This doesn't work
    if(allTree.view) allTree.view.selection.select(-1); // So we use XPCOM

    prefbarActiveTree = enabledTree;
    prefbarUpdateButtons();

    // Disable Enable button
    //enableButton.disabled = true;
}

function prefbarInitAllTree() {
    //goPrefBar.dump("initall");
    // Remove selection from other tree
    // enabledTree.currentIndex == -1;      // This doesn't work
    if (enabledTree.view) enabledTree.view.selection.select(-1);  // So we use XPCOM

    prefbarActiveTree = allTree;
    prefbarUpdateButtons();

    // Disable Disable button
    //disableButton.disabled = true;
}

function prefbarUpdateButtons() {
    var tree = prefbarActiveTree;
    if (!tree) return;
    var selection = tree.view.selection.currentIndex;
    //var selectionCount = tree.treeBoxObject.selection.count; -> Not longer supported in Mozilla 1.8a1
    var selectionCount = tree.view.selection.count;

    // enable/disable buttons depending on the number of items selected
    var isNothingSelected = (selectionCount == 0);
    if (isNothingSelected) {
	enableButton.disabled = true;
	disableButton.disabled = true;
    }
    else {
	enableButton.disabled = (prefbarActiveTree == enabledTree);
	disableButton.disabled = (prefbarActiveTree == allTree);
    }
    deleteButton.disabled = isNothingSelected;
    exportButton.disabled = isNothingSelected;
    // can only edit one item at once
    // xxx we should probably disable the Edit button if a separator is selected
    editButton.disabled = (selectionCount != 1);

    // Set Up button
    // If we are selecting the very first item or more than one Item is selected
    moveUpButton.disabled = (selection==0 || selectionCount!=1);

    // Set down button
    // If we are selecting the very last item or more than one Item is selected
    moveDownButton.disabled = (selection==tree.view.rowCount-1 || selectionCount!=1);
}


function prefbarItemEnable() {
    var REnabled = RDF.GetResource("urn:prefbar:browserbuttons:enabled");
    var RDisabled = RDF.GetResource("urn:prefbar:browserbuttons:disabled");

    var CEnabled = Components.classes["@mozilla.org/rdf/container;1"].createInstance(Components.interfaces.nsIRDFContainer);
    CEnabled.Init(mDatasource, REnabled);
    var CDisabled = Components.classes["@mozilla.org/rdf/container;1"].createInstance(Components.interfaces.nsIRDFContainer);
    CDisabled.Init(mDatasource, RDisabled);

    var destid = 1;
    var selections = prefbarGetTreeSelections(allTree);
    for (var i = 0; i <= selections.length - 1; i++) {
        var itemId = selections[i];
        var RitemId = RDF.GetResource(itemId);
        CDisabled.RemoveElement(RitemId, true);
        CEnabled.InsertElementAt(RitemId, destid, true);
        destid++;
    }
}


function prefbarItemDisable() {
    var REnabled = RDF.GetResource("urn:prefbar:browserbuttons:enabled");
    var RDisabled = RDF.GetResource("urn:prefbar:browserbuttons:disabled");

    var CEnabled = Components.classes["@mozilla.org/rdf/container;1"].createInstance(Components.interfaces.nsIRDFContainer);
    CEnabled.Init(mDatasource, REnabled);
    var CDisabled = Components.classes["@mozilla.org/rdf/container;1"].createInstance(Components.interfaces.nsIRDFContainer);
    CDisabled.Init(mDatasource, RDisabled);

    var destid = 1;
    var selections = prefbarGetTreeSelections(enabledTree);
    for (var i = 0; i <= selections.length - 1; i++) {
        var itemId = selections[i];
        var RitemId = RDF.GetResource(itemId);
        CEnabled.RemoveElement(RitemId, true);
        CDisabled.InsertElementAt(RitemId, destid, true);
        destid++;
    }
}

function getCNameById(id) {
    var item = document.getElementById(id);
    item = item.parentNode.parentNode;
    if (item.localName == "treeitem")
        return item.id;
    else
        return item.ref;
}

function prefbarItemMoveUp() {
    var selection = prefbarActiveTree.view.selection.currentIndex;
    var sourceId = prefbarActiveTree.contentView.getItemAtIndex(selection).getAttribute("id");

    var RCName = RDF.GetResource(getCNameById(sourceId));
    var RItem = RDF.GetResource(sourceId);

    var container = Components.classes["@mozilla.org/rdf/container;1"].createInstance(Components.interfaces.nsIRDFContainer);
    container.Init(mDatasource, RCName);

    // selection-id is zero based, container-ids are one-based!
    var cid = selection + 1
    container.RemoveElementAt(cid, true);
    container.InsertElementAt(RItem, cid - 1, true);

    prefbarActiveTree.view.selection.select(selection - 1);
    prefbarActiveTree.focus();
}

function prefbarItemMoveDown() {
    var selection = prefbarActiveTree.view.selection.currentIndex;
    var sourceId = prefbarActiveTree.contentView.getItemAtIndex(selection).getAttribute("id");

    var RCName = RDF.GetResource(getCNameById(sourceId));
    var RItem = RDF.GetResource(sourceId);

    var container = Components.classes["@mozilla.org/rdf/container;1"].createInstance(Components.interfaces.nsIRDFContainer);
    container.Init(mDatasource, RCName);

    // selection-id is zero based, container-ids are one-based!
    var cid = selection + 1;
    container.RemoveElementAt(cid, true);
    container.InsertElementAt(RItem, cid + 1, true);

    prefbarActiveTree.view.selection.select(selection + 1);
    prefbarActiveTree.focus();
}

function prefbarItemNew() {
    var item1 = enabledTree.contentView.getItemAtIndex(0);
    window.openDialog("chrome://prefbar/content/newItem/newItem.xul", "newItemDialog", "chrome,titlebar,dialog,modal,resizable");
    var item2 = enabledTree.contentView.getItemAtIndex(0);

    if (item1 != item2) {
        enabledTree.focus();
        enabledTree.view.selection.select(0);
    }
}

function prefbarItemEdit() {
    var selection = prefbarActiveTree.view.selection.currentIndex;
    var selItemId = prefbarActiveTree.treeBoxObject.treeBody.childNodes[selection].getAttribute("id");

    window.openDialog("chrome://prefbar/content/newItem/editItem.xul", "editItemDialog", "chrome,titlebar,dialog,modal,resizable", selItemId);
}


function prefbarItemDelete() {
    if(!confirm(goPrefBar.GetString("preferencesPanelStrings", "questiondelete"))) return;

    var selections = prefbarGetTreeSelections(prefbarActiveTree);
    for(var i = 0; i < selections.length; i++) {
        var selItemId = selections[i];

        var RselItem = RDF.GetResource(selItemId);
        var Rbuttons = RDF.GetResource(getCNameById(selItemId));

        var properties = mDatasource.ArcLabelsOut(RselItem);

        while(properties.hasMoreElements()) {
            var Rproperty = properties.getNext();

            var Lvalue = mDatasource.GetTarget(RselItem, Rproperty, true);
            mDatasource.Unassert(RselItem, Rproperty, Lvalue);
        }

        var container = Components.classes["@mozilla.org/rdf/container;1"].getService(Components.interfaces.nsIRDFContainer);
        container.Init(mDatasource, Rbuttons);

        container.RemoveElement(RselItem, true);
    }
}

function prefbarGetTreeSelections(tree) {
    var selections = [];
    var select = tree.view.selection;
    if (select) {
        var count = select.getRangeCount();
        var min = new Object();
        var max = new Object();
        for (var i=0; i<count; i++) {
            select.getRangeAt(i, min, max);
            for (var k=min.value; k<=max.value; k++) {
                if (k != -1) {
                    selections[selections.length] = tree.contentView.getItemAtIndex(k).getAttribute("id");
                }
            }
        }
    }
    return selections;
}

function prefbarWriteToDisk() {
    goPrefBar.RDF.mDatasource.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();
}

function prefbarItemExport() {
    var nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);

    fp.init(window, "Export Buttons to File:", nsIFilePicker.modeSave);
    fp.appendFilter("ButtonCollections", "*.btn");
    //fp.appendFilters(nsIFilePicker.filterAll);
    fp.defaultExtension = "btn";

    var res=fp.show();
    if (res==nsIFilePicker.returnOK || res == nsIFilePicker.returnReplace) {
	var selections = prefbarGetTreeSelections(prefbarActiveTree);
	var toexport = new Array;
	for (var i = 0; i <= selections.length - 1; i++) {
	    var itemId = selections[i];
			
	    toexport.push(itemId);
	}

	// Call the exporter
	goPrefBar.ImpExp.Export(window, toexport, fp.fileURL.spec);

	delete toexport;
    }
}


function prefbarItemImport() {
    var nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);

    fp.init(window, "Import Buttons from File:", nsIFilePicker.modeOpen);
    fp.appendFilter("ButtonCollections", "*.btn");
    //fp.appendFilters(nsIFilePicker.filterAll);
    fp.defaultExtension = "btn";

    var res=fp.show();
    if (res==nsIFilePicker.returnOK) {
        goPrefBar.ImpExp.Import(window, fp.fileURL.spec);
    }
}


var dragService = Components.classes["@mozilla.org/widget/dragservice;1"].getService().QueryInterface(Components.interfaces.nsIDragService);
var nsIDragService = Components.interfaces.nsIDragService;

// It's much easier to move drag data in an array! This is no problem,
// as we don't support drag&drop to anything outside this preferences window!
var gDragArray;

function BeginDragTree(event, tree) {
    goPrefBar.dump("BeginDragTree");

    if (event.originalTarget.localName != "treechildren")
	return false;

    var row = tree.treeBoxObject.getRowAt(event.clientX, event.clientY);
    goPrefBar.dump("BeginDragTree: row: " + row);
    if (row == -1)
	return false;

    gDragArray = new Array();

    var selection = prefbarGetTreeSelections(tree);
    for (var i = 0; i < selection.length; ++i ) {
        var curItem = selection[i];
        var ItemArray = new Array(curItem, getCNameById(curItem));
        gDragArray.push(ItemArray);
    }


    var region = null;
/*  try {
	region = Components.classes["@mozilla.org/gfx/region;1"].createInstance(Components.interfaces.nsIScriptableRegion);
	region.init();
	var obo = tree.treeBoxObject;
	var bo = obo.treeBody.boxObject;
	var sel= tree.view.selection;

	var rowX = bo.x;
	var rowY = bo.y;
	var rowHeight = obo.rowHeight;
	var rowWidth = bo.width;

	//add a rectangle for each visible selected row
	for (var i = obo.getFirstVisibleRow(); i <= obo.getLastVisibleRow(); i ++) {
	    if (sel.isSelected(i))
		region.unionRect(rowX, rowY, rowWidth, rowHeight);
	    rowY = rowY + rowHeight;
	}
      
	//and finally, clip the result to be sure we don't spill over...
	if(!region.isEmpty())
	    region.intersectRect(bo.x, bo.y, bo.width, bo.height);
    } catch(ex) {
	dump("Error while building selection region: " + ex + "\n");
	region = null;
    }
*/
  
    var transArray = Components.classes["@mozilla.org/supports-array;1"].createInstance(Components.interfaces.nsISupportsArray);
    if ( !transArray ) 
	return false;
    
    var flavor = "text/x-moz-prefbar-button";

    var trans = Components.classes["@mozilla.org/widget/transferable;1"].createInstance(Components.interfaces.nsITransferable);
    if (!trans) 
        return false;

    var genTextData = Components.classes["@mozilla.org/supports-string;1"].createInstance(Components.interfaces.nsISupportsString);
    if (!genTextData) 
        return false;

    trans.addDataFlavor(flavor);

    // We just drag the word "prefbar" the usual way. The real data is in gDragArray.
    var string = "prefbar"
    genTextData.data = string;

    trans.setTransferData ( flavor, genTextData, string.length * 2 );  // doublebyte byte data

    // put it into the transferable as an |nsISupports|
    var genTrans = trans.QueryInterface(Components.interfaces.nsISupports);
    transArray.AppendElement(genTrans);

    dragService.invokeDragSession ( event.target, transArray, region, nsIDragService.DRAGDROP_ACTION_COPY + nsIDragService.DRAGDROP_ACTION_MOVE );

    return false;  // don't propagate the event if a drag has begun
}

function DragOverTree(event) {
    var dragSession = dragService.getCurrentSession();
    if (!dragSession)
        return false;

    if (!dragSession.isDataFlavorSupported("text/x-moz-prefbar-button"))
	return false;

    dragSession.canDrop = true;

    return true;
}

function DropOnTree(event, tree) {
    goPrefBar.dump("DropOnTree");
    var dragSession = dragService.getCurrentSession();
    if (! dragSession )
        return false;
    
    // The drop procedure may take really long (depending on how much items have to be moved)...
    window.setCursor("wait");

    var idPlaceBefore = null;
    var PlaceAfter = false;

    var tbo = tree.treeBoxObject;
    if (tbo.treeBody && tbo.treeBody.childNodes.length > 0) {
	var row = tbo.getRowAt(event.clientX, event.clientY);
	goPrefBar.dump("DropOnTree: Dropped On Row: " + row);
	if (row != -1) {
	    var rowHeight = tbo.rowHeight;
	    var rowY = tbo.treeBody.boxObject.y;
	    var firstRow = tbo.getFirstVisibleRow();
	    var midItemPos = rowY + rowHeight * (row - firstRow) + rowHeight / 2;

	    idPlaceBefore = tbo.treeBody.childNodes[row].getAttribute("id");
	    
	    if (event.clientY > midItemPos) {
		PlaceAfter = true;
	    }
	}
	else {
	    goPrefBar.dump("DropOnTree: Dropped after last item");
	    var lastRow = tbo.treeBody.childNodes.length - 1;
	    idPlaceBefore = tbo.treeBody.childNodes[lastRow].getAttribute("id");
	    PlaceAfter = true;
	}
    }
    else {
	goPrefBar.dump("DropOnTree: Dropped On Empty Tree");
    }

    var RDFCU = Components.classes['@mozilla.org/rdf/container-utils;1'].getService();
    RDFCU = RDFCU.QueryInterface(Components.interfaces.nsIRDFContainerUtils);

    // Get target sequence and container
    var targetseq;
    if (!idPlaceBefore)
        targetseq = tree.ref;
    else
        targetseq = getCNameById(idPlaceBefore);
    var targetcontainer = RDFCU.MakeSeq(mDatasource, RDF.GetResource(targetseq));

    // first run: Kill all items from old seq
    for (var i = 0; i < gDragArray.length; i++) {
        var curitem = gDragArray[i];
        var container = RDFCU.MakeSeq(mDatasource, RDF.GetResource(curitem[1]));
        container.RemoveElement(RDF.GetResource(curitem[0]), true);
    }

    // Now get target position.
    var newpos;
    if (!idPlaceBefore)
        newpos = 1;
    else {
        newpos = targetcontainer.IndexOf(RDF.GetResource(idPlaceBefore));
        if (newpos == -1)
            newpos = 1;
        else
            if (PlaceAfter) newpos++;
    }

    // Second run: Add all items to new seq on their new position
    for (var i = 0; i < gDragArray.length; i++) {
        var curitem = gDragArray[i];
        targetcontainer.InsertElementAt(RDF.GetResource(curitem[0]), newpos, true);
        newpos++;
    }

    // Focus target tree
    tree.focus();

    // Add selection for newly added items
    for (var index = 0; index < tbo.treeBody.childNodes.length; index++) {
        if (tbo.treeBody.childNodes[index].getAttribute("id") == gDragArray[0][0]) {
            tree.view.selection.rangedSelect(index, index + gDragArray.length - 1, false);
            break;
        }
    }

    window.setCursor("auto");
    return true;
}
